from __future__ import unicode_literals
from catsnap import Client, HASH_KEY
from boto.dynamodb.batch import BatchList

MAX_ITEMS_TO_REQUEST = 99

def get_item_batch(tag_names, table_name, attributes_to_get):
    if not tag_names:
        raise StopIteration

    tag_names = list(set(tag_names))

    unprocessed_keys = tag_names[MAX_ITEMS_TO_REQUEST:]
    tag_names = tag_names[:MAX_ITEMS_TO_REQUEST]

    dynamo = Client().get_dynamodb()
    table = Client().table(table_name)
    batch_list = BatchList(dynamo)
    batch_list.add_batch(table, tag_names,
            attributes_to_get=attributes_to_get + [HASH_KEY])
    response = dynamo.batch_get_item(batch_list)
    items = response['Responses'][table.name]['Items']
    if response['UnprocessedKeys'] \
            and table.name in response['UnprocessedKeys']:
        for key in response['UnprocessedKeys'][table.name]['Keys']:
            unprocessed_keys.append(key['HashKeyElement'])

    for item in items:
        yield item
    for item in get_item_batch(unprocessed_keys, table_name, attributes_to_get):
        yield item




from __future__ import unicode_literals

from catsnap.singleton import Singleton
from catsnap.config.base import Config
from catsnap.config.argument_config import ArgumentConfig
from catsnap.config.env_config import EnvConfig
from catsnap.config.file_config import FileConfig

class MetaConfig(object):
    _file_config = {}
    _env_config = {}
    _argument_config = {}
    _defaults = {'extension': False}

    def __init__(self, include_arg_config=False):
        self._file_config = FileConfig()
        self._env_config = EnvConfig()
        if include_arg_config:
            self._argument_config = ArgumentConfig()

    def __getitem__(self, item):
        for subconfig in (self._argument_config,
                          self._file_config,
                          self._env_config,
                          self._defaults):
            if item in subconfig:
                return subconfig[item]

        raise KeyError("Couldn't find any setting at all for '%s'. You'll need "
                "to supply it in some way--try `catsnap config`, or see the "
                "docs for other ways to supply a setting." % item)

    def __contains__(self, item):
        return any(map(lambda x: item in x, [self._argument_config,
                                             self._file_config,
                                             self._env_config,
                                             self._defaults]))

    def __getattr__(self, item):
        if item not in Config.ALL_SETTINGS:
            raise AttributeError(item)
        return self[item]



from __future__ import unicode_literals

from catsnap import HASH_KEY
from catsnap.batch import get_item_batch
import json

def get_images(filenames):
    for item in get_item_batch(filenames, 'image', ['tags']):
        yield {'filename': item[HASH_KEY],
               'tags': json.loads(item['tags'])}



from __future__ import unicode_literals

import ConfigParser
import os
import sys
import getpass

from catsnap.config.base import Config

LEGACY_CREDENTIALS_FILE = os.path.join(os.environ['HOME'], '.boto')
CONFIG_FILE = os.path.join(os.environ['HOME'], '.catsnap')
BUCKET_BASE = 'catsnap'

def _input(*args, **kwargs):
    return raw_input(*args, **kwargs)


class FileConfig(Config):
    def __init__(self):
        self._parser = ConfigParser.ConfigParser()
        self._parser.read(LEGACY_CREDENTIALS_FILE)
        self._parser.read(CONFIG_FILE)
        self.setting_definitions = self._setting_definitions()

        for setting in self.ALL_SETTINGS:
            if setting not in self.setting_definitions:
                raise AttributeError(setting)

    def __getitem__(self, item):
        section = 'catsnap'
        if item in self.CREDENTIAL_SETTINGS:
            section = 'Credentials'

        getter = getattr(self._parser,
                self.setting_definitions[item].parser_getter)
        return getter(section, item)

    def __contains__(self, item):
        section = 'catsnap'
        if item in self.CREDENTIAL_SETTINGS:
            section = 'Credentials'
        return self._parser.has_option(section, item)

    def collect_settings(self, settings_to_get=None):
        if not settings_to_get:
            settings_to_get = self.CLIENT_SETTINGS

        for setting in settings_to_get:
            self._get_setting(setting)

        with open(CONFIG_FILE, 'w') as config_file:
            self._parser.write(config_file)

    def _get_setting(self, setting_name):
        setting = self.setting_definitions[setting_name]

        if not self._parser.has_section(setting.section):
            self._parser.add_section(setting.section)

        has_setting = self._parser.has_option(setting.section, setting.name)
        use_default = ''
        if setting.global_default:
            use_default = setting.override_message % setting.global_default
        if has_setting:
            try:
                use_default = setting.override_message % \
                        self._parser.get(setting.section, setting.name)
            except TypeError:
                use_default = setting.override_message
        setting_value = setting.read_method(setting.message % use_default)
        if has_setting and not setting_value:
            setting_value = self._parser.get(setting.section, setting.name)
        setting_value = setting_value or setting.global_default
        self._parser.set(setting.section, setting.name, setting_value)

    @classmethod
    def _setting_definitions(cls):
        return {setting.name: setting for setting in (
            FileSetting(
                section='Credentials',
                name='aws_access_key_id',
                message='Enter your access key id%s: ',
                override_message=" (leave blank to keep using '%s')"),
            FileSetting(
                section='Credentials',
                name='aws_secret_access_key',
                message='Enter your secret access key%s: ',
                override_message=" (leave blank to keep using "
                                 "what you had before)",
                read_method=getpass.getpass),
            FileSetting(
                section='catsnap',
                name='bucket',
                message="Please name your bucket%s: ",
                global_default='%s-%s' % (BUCKET_BASE, os.environ.get('USER')),
                override_message=" (leave blank to use '%s')"),
            FileSetting(
                section='catsnap',
                name='extension',
                message='Would you like to print a fake file extension '
                        'on urls%s? ',
                override_message="(leave blank to keep using '%s')",
                parser_getter='getboolean'),
            FileSetting(
                section='catsnap',
                name='api_host',
                message='Enter the host for your catsnap api%s: ',
                override_message=" (leave blank to keep using '%s'"),
            FileSetting(
                section='catsnap',
                name='api_key',
                message='Enter your catsnap api key%s: ',
                override_message=" (leave blank to keep using "
                                 "what you had before)",
                read_method=getpass.getpass),
            FileSetting(
                section='catsnap',
                name='owner_id',
                message='Enter your openid url%s: ',
                override_message=" (leave blank to keep using '%s')"),
            FileSetting(
                section='catsnap',
                name='owner_email',
                message='Enter your email address%s: ',
                override_message=" (leave blank to keep using '%s')"),
        )}


class FileSetting(object):
    section = NotImplemented
    name = NotImplemented
    message = NotImplemented
    override_message = NotImplemented
    read_method = None
    global_default = None
    parser_getter = 'get'

    def __init__(self, **kwargs):
        for attr, value in kwargs.iteritems():
            if hasattr(self, attr):
                setattr(self, attr, value)
            else:
                raise TypeError(attr)
        if 'read_method' not in kwargs:
            self.read_method = _input

        for attr in dir(self):
            if getattr(self, attr) is NotImplemented:
                raise NotImplementedError(attr)




from __future__ import unicode_literals

from catsnap import Client, HASH_KEY
from boto.dynamodb.batch import BatchWriteList, BatchList
from boto.dynamodb.item import Item
from catsnap.batch import get_item_batch
import json

def get_tags(tag_names):
    for item in get_item_batch(tag_names, 'tag', ['filenames']):
        yield { 'tag': item[HASH_KEY],
                'filenames': json.loads(item['filenames'])}

def add_image_to_tags(filename, tag_names):
    items = list(get_item_batch(tag_names, 'tag', ['filenames']))
    for item in items:
        filenames = json.loads(item['filenames'])
        filenames.append(filename)
        item['filenames'] = json.dumps(filenames)

    table = Client().table('tag')
    for new_tag in set(tag_names) - set(x[HASH_KEY] for x in items):
        items.append(table.new_item(hash_key = new_tag,
                attrs={'filenames': json.dumps([filename])}))

    _submit_items(table, items)

def _submit_items(table, items):
    dynamo = Client().get_dynamodb()
    write_list = BatchWriteList(dynamo)
    write_list.add_batch(table, puts=items)
    response = write_list.submit()
    if 'UnprocessedItems' in response \
            and table.name in response['UnprocessedItems']:
        unprocessed_keys = set(x['PutRequest']['Item'][HASH_KEY]
                for x in response['UnprocessedItems'][table.name])
        unprocessed_items = filter(
                lambda x: x[HASH_KEY] in unprocessed_keys, items)
        _submit_items(table, unprocessed_items)



from __future__ import unicode_literals
import os

from catsnap.config.base import Config

class EnvConfig(Config):
    def __getitem__(self, item):
        env_var = 'CATSNAP_' + item.upper()
        return os.environ[env_var]

    def __contains__(self, item):
        env_var = 'CATSNAP_' + item.upper()
        return env_var in os.environ



import os
from catsnap.web import app

if __name__ == '__main__':
    port = int(os.environ.get('PORT', 5000))
    app.run(host='0.0.0.0',
            port=port,
            debug=os.environ.get('CATSNAP_DEBUG', False))



from __future__ import unicode_literals

class Config(object):
    CREDENTIAL_SETTINGS = ('aws_access_key_id', 'aws_secret_access_key')
    ALL_SETTINGS = CREDENTIAL_SETTINGS + ('bucket',
                                          'extension',
                                          'owner_id',
                                          'owner_email',
                                          'api_host',
                                          'api_key')
    CLIENT_SETTINGS = ('extension', 'api_host', 'api_key')

    def __getitem__(self, item): raise NotImplementedError
    def __contains__(self, item): raise NotImplementedError



from __future__ import unicode_literals
import boto
import os

from catsnap.config import MetaConfig
from catsnap.config.file_config import FileConfig
from catsnap.config.env_config import EnvConfig
from catsnap.singleton import Singleton
from boto.exception import DynamoDBResponseError, S3CreateError
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

#This really oughtta be, like, the tablename or something, but I screwed up, so
#now there're existing catsnap installs that use this schema. Sucks :(
#So yeah every table is keyed on an attribute called 'tag'
HASH_KEY = 'tag'

class Client(Singleton):
    _tables = {}
    _bucket = None
    _config = None

    _dynamo_connection = None
    _s3_connection = None
    _engine = None
    _session = None

    def config(self, new_config=None):
        if new_config is not None:
            self._config = new_config
        if self._config is None:
            self._config = MetaConfig()
        return self._config

    def bucket(self):
        if not self._bucket:
            s3 = self.get_s3()
            bucket_name = self.config().bucket
            self._bucket = s3.get_bucket(bucket_name)
        return self._bucket

    def table(self, table_name):
        table_prefix = self.config().bucket
        table_name = '%s-%s' % (table_prefix, table_name)

        if table_name not in self._tables:
            dynamo = self.get_dynamodb()
            self._tables[table_name] = dynamo.get_table(table_name)
        return self._tables[table_name]

    def get_dynamodb(self):
        if not self._dynamo_connection:
            self._dynamo_connection = boto.connect_dynamodb(
                    aws_access_key_id=self.config().aws_access_key_id,
                    aws_secret_access_key=self.config().aws_secret_access_key)
        return self._dynamo_connection

    def get_s3(self):
        if not self._s3_connection:
            self._s3_connection = boto.connect_s3(
                    aws_access_key_id=self.config().aws_access_key_id,
                    aws_secret_access_key=self.config().aws_secret_access_key)
        return self._s3_connection

    def session(self):
        if not self._engine:
            self._engine = create_engine(os.environ['DATABASE_URL'])
        if not self._session:
            self._session = sessionmaker(bind=self._engine)()
        return self._session




from __future__ import unicode_literals

import sys
import argparse

from catsnap.config.base import Config

class ArgumentConfig(Config):
    def __init__(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('-e', '--extension', action='store_true',
                default=None,
                help="Append #.gif to urls (i.e. for pasting in Campfire)")
        parser.add_argument('--no-extension', action='store_false',
                dest='extension', default=None,
                help="Do no append #.gif to urls")
        parser.add_argument('--api-host', help="The host where your catsnap "
                                               "api can be found")
        parser.add_argument('--api-key', help="The the key for "
                                              "your catsnap api")
        parser.add_argument('arg', nargs='*')

        #We're manually grabbing sys.argv so the tests can easily mock it
        #We need to drop the first item because argparse will see it as an
        #argument, rather than as the script-name
        argv = sys.argv[1:]
        self._args = parser.parse_args(argv)

        for setting in self.CLIENT_SETTINGS:
            if not hasattr(self._args, setting):
                raise AttributeError(setting)

    def __getitem__(self, item):
        try:
            argument = getattr(self._args, item)
            if argument is None:
                raise KeyError(item)
            return argument
        except AttributeError:
            raise KeyError(item)

    def __contains__(self, item):
        return hasattr(self._args, item) and \
                getattr(self._args, item) is not None



#!/usr/bin/python

from setuptools import setup

setup(name="catsnap",
      version="3.0.1",
      description="catalog and store funny pictures",
      author="Andrew Lorente",
      author_email="andrew.lorente@gmail.com",
      url="github.com/andrewlorente/catsnap",
      packages=['catsnap',
                'catsnap.document',
                'catsnap.config',
                'catsnap.batch'],
      scripts=['scripts/catsnap'],
      install_requires=[
          "Flask==0.9",
          "gunicorn==0.14.6",
          "boto==2.5.2",
          "requests==0.13.2",
          "argparse==1.2.1",
          "flask-openid==1.1.1",
          "psycopg2==2.4.6",
          "sqlalchemy==0.8.0b2",
          "yoyo-migrations==4.1.6",

          "mock==0.8",
          "nose==1.1.2",
          ],
      )
